#include "xprt.h"

/* ------------------------------- NewNode ---------------------------- */

Node *NewNode(void)
{
   Node *node;

   node = (Node*)(SafeMalloc(sizeof(Node)));

   node->gate = NULL;
   node->xprt = NULL;

   return (node);
}

/* ------------------------------- NewXprt ---------------------------- */

Xprt *NewXprt(void)
{
   Xprt *xprt;
   CoordinateNumber i, j;

   xprt = (Xprt*)(SafeMalloc(sizeof(Xprt)));

   xprt->wt = NewMatrix(gDimY, gDimX);
   for (i=0; i<gDimY; ++i)
      for (j=0; j<gDimX; ++j)
         xprt->wt[i][j] = kWtScale * Rand1();

   switch (gXprtModel) {
      case gaussian:
         xprt->var = NewVector(gNumPts);
         xprt->phi = 1.0;
         break;
      case bernoulli:
         xprt->var = NULL;
         xprt->phi = 1.0;
         break;
   }

   return (xprt);
}

/* ------------------------------- NewGate ---------------------------- */

Gate *NewGate(void)
{
   Gate *gate;
   CoordinateNumber n;

   gate = (Gate*)(SafeMalloc(sizeof(Gate)));

   gate->wt = NewVector(gDimX);
   for (n=0; n<gDimX; ++n)
      gate->wt[n] = kWtScale * Rand1();

   gate->h = NewVector(gNumPts);

   gate->node1 = NewNode();
   gate->node2 = NewNode();

   return (gate);
}

/* ------------------------------ AddLayer ---------------------------- */

/* AddLayer splits "nSplits" experts in the bottom layer into gating    */
/* networks, thus creating a new layer of (2*nSplits) experts.          */

void AddLayer(Node *node, int *nSplits)
{
   CoordinateNumber n;
   Gate *gate;
   Xprt *xprt;

   if ((*nSplits)==0)
      return;

   if (gate=IsGate(node))
   {
      AddLayer(gate->node1, nSplits);
      AddLayer(gate->node2, nSplits);
   }
   else
   if (xprt=IsXprt(node))
   {
      node->xprt = NULL;
      node->gate = NewGate();

      gate = node->gate;
      gate->node1->xprt = xprt;
      gate->node2->xprt = NewXprt();

      --(*nSplits);
   }
}

/* ------------------------------- NewMatrix ---------------------------- */

Matrix NewMatrix(int nRows, int nColumns)
{
   Matrix mx;

   mx = (Matrix)(SafeCalloc(nRows, sizeof(double*)));
   while (nRows--)
      mx[nRows] = (double*)(SafeCalloc(nColumns, sizeof(double)));

   return (mx);
}

/* ------------------------------- NewVector ---------------------------- */

Vector NewVector(int n)
{
   return ((Vector)(SafeCalloc(n, sizeof(double))));
}

/* ---------------------------------------------------------------------- */
